@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2014-2018 Tim Ruehsen
+ * Copyright(c) 2014-2024 Tim Ruehsen
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -82,6 +82,8 @@ typedef SSIZE_T ssize_t;
 
 #ifdef HAVE_ALLOCA_H
 #	include <alloca.h>
+#elif defined _WIN32
+#	include <malloc.h>
 #endif
 
 #ifdef WITH_LIBICU
@@ -102,9 +104,13 @@ typedef SSIZE_T ssize_t;
 #	include <unistr.h>
 #endif
 
-#ifndef WINICONV_CONST
-#  define WINICONV_CONST
+#ifdef WINICONV_CONST
+#  define ICONV_CONST WINICONV_CONST
 #endif
+#ifndef ICONV_CONST
+#  define ICONV_CONST
+#endif
+
 
 #include <libpsl.h>
 
@@ -129,7 +135,7 @@ typedef SSIZE_T ssize_t;
 
 typedef struct {
 	char
-		label_buf[48];
+		label_buf[128];
 	const char *
 		label;
 	unsigned short
@@ -166,7 +172,7 @@ struct psl_ctx_st {
 };
 
 /* include the PSL data generated by psl-make-dafsa */
-#if defined(BUILTIN_GENERATOR_LIBICU) || defined(BUILTIN_GENERATOR_LIBIDN2) || defined(BUILTIN_GENERATOR_LIBIDN)
+#ifdef ENABLE_BUILTIN
 #include "suffixes_dafsa.h"
 #else
 static const unsigned char kDafsa[] = "";
@@ -568,16 +574,14 @@ static int domain_to_punycode(const char *domain, char *out, size_t outsize)
 	punycode_uint input[256];
 	const char *label, *e;
 
-	for (e = label = domain; e; label = e + 1) {
+	for (e = label = domain; e;) {
 		e = strchr(label, '.');
 		labellen = e ? (size_t) (e - label) : strlen(label);
-		/* printf("s=%s inlen=%zd\n", label, labellen); */
 
 		if (mem_is_ascii(label, labellen)) {
 			if (outlen + labellen + (e != NULL) >= outsize)
 				return 1;
 
-			/* printf("outlen=%zd labellen=%zd\n", outlen, labellen); */
 			memcpy(out + outlen, label, labellen);
 			outlen += labellen;
 		} else {
@@ -592,15 +596,16 @@ static int domain_to_punycode(const char *domain, char *out, size_t outsize)
 			memcpy(out + outlen, "xn--", 4);
 			outlen += 4;
 
-			labellen = outsize - outlen;
-			/* printf("n=%zd space_left=%zd\n", n, labellen); */
+			labellen = outsize - outlen - (e != NULL) - 1; // -1 to leave space for the trailing \0
 			if (punycode_encode(inputlen, input, &labellen, out + outlen))
 				return 1;
 			outlen += labellen;
 		}
 
-		if (e)
+		if (e) {
+			label = e + 1;
 			out[outlen++] = '.';
+		}
 		out[outlen] = 0;
 	}
 
@@ -838,8 +843,11 @@ static int is_public_suffix(const psl_ctx_t *psl, const char *domain, int type)
 	suffix.nlabels = 1;
 
 	for (p = domain; *p; p++) {
-		if (*p == '.')
+		if (*p == '.') {
+			if (suffix.nlabels == 255) // weird input, avoid 8bit overflow
+				return 0;
 			suffix.nlabels++;
+		}
 		else if (*((unsigned char *)p) >= 128)
 			need_conversion = 1; /* in case domain is non-ascii we need a toASCII conversion */
 	}
@@ -934,13 +942,11 @@ static int is_public_suffix(const psl_ctx_t *psl, const char *domain, int type)
 		}
 
 		if ((suffix.label = strchr(suffix.label, '.'))) {
-			int pos;
-
 			suffix.label++;
 			suffix.length = strlen(suffix.label);
 			suffix.nlabels--;
 
-			rule = vector_get(psl->suffixes, (pos = vector_find(psl->suffixes, &suffix)));
+			rule = vector_get(psl->suffixes, vector_find(psl->suffixes, &suffix));
 
 			if (rule) {
 				/* check for correct rule type */
@@ -1364,7 +1370,7 @@ void psl_free(psl_ctx_t *psl)
  */
 const psl_ctx_t *psl_builtin(void)
 {
-#if defined(BUILTIN_GENERATOR_LIBICU) || defined(BUILTIN_GENERATOR_LIBIDN2) || defined(BUILTIN_GENERATOR_LIBIDN)
+#ifdef ENABLE_BUILTIN
 	return &builtin_psl;
 #else
 	return NULL;
@@ -1622,6 +1628,9 @@ static int isip(const char *hostname)
  *
  * Use helper function psl_str_to_utf8lower() for normalization of @hostname and @cookie_domain.
  *
+ * Hint for Windows users:
+ * Please make sure the calling application has called WSAStartup() before calling psl_is_cookie_domain_acceptable().
+ *
  * Examples:
  * 1. Cookie domain 'example.com' would be acceptable for hostname 'www.example.com',
  * but '.com' or 'com' would NOT be acceptable since 'com' is a public suffix.
@@ -1830,7 +1839,7 @@ out:
 				if (!dst) {
 					ret = PSL_ERR_NO_MEM;
 				}
-				else if (iconv(cd, (WINICONV_CONST char **)&tmp, &tmp_len, &dst_tmp, &dst_len_tmp) != (size_t)-1
+				else if (iconv(cd, (ICONV_CONST char **)&tmp, &tmp_len, &dst_tmp, &dst_len_tmp) != (size_t)-1
 					&& iconv(cd, NULL, NULL, &dst_tmp, &dst_len_tmp) != (size_t)-1)
 				{
 					/* start size for u8_tolower internal memory allocation.
